@@ -88,7 +88,7 @@ char *opt_ibx_incremental_history_name = NULL;
 char *opt_ibx_incremental_history_uuid = NULL;
 
 char *opt_ibx_user = NULL;
-char *opt_ibx_password = NULL;
+const char *opt_ibx_password = NULL;
 char *opt_ibx_host = NULL;
 char *opt_ibx_defaults_group = NULL;
 char *opt_ibx_socket = NULL;
@@ -112,6 +112,8 @@ bool ibx_partial_backup = false;
 char *ibx_position_arg = NULL;
 char *ibx_backup_directory = NULL;
 
+extern bool xb_opt_destroy_password;
+
 /* copy of proxied xtrabackup options */
 my_bool ibx_xb_close_files;
 const char *ibx_xtrabackup_compress_alg;
@@ -206,8 +208,7 @@ enum innobackupex_options
 	OPT_STREAM,
 	OPT_TABLES_FILE,
 	OPT_THROTTLE,
-	OPT_USE_MEMORY,
-	OPT_INNODB_FORCE_RECOVERY,
+	OPT_USE_MEMORY
 };
 
 ibx_mode_t ibx_mode = IBX_MODE_BACKUP;
@@ -625,16 +626,6 @@ static struct my_option ibx_long_options[] =
 	 0, GET_LL, REQUIRED_ARG, 100*1024*1024L, 1024*1024L, LONGLONG_MAX, 0,
 	 1024*1024L, 0},
 
-	{"innodb-force-recovery", OPT_INNODB_FORCE_RECOVERY,
-	 "This option starts up the embedded InnoDB instance in crash "
-	 "recovery mode to ignore page corruption; should be used "
-	 "with the \"--apply-log\" option, in emergencies only. The "
-	 "default value is 0. Refer to \"innodb_force_recovery\" server "
-	 "system variable documentation for more details.",
-	 (uchar*)&xtrabackup_innodb_force_recovery,
-	 (uchar*)&xtrabackup_innodb_force_recovery,
-	 0, GET_ULONG, OPT_ARG, 0, 0, SRV_FORCE_IGNORE_CORRUPT, 0, 0, 0},
-
 	{ 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -680,7 +671,6 @@ innobackupex [--compress] [--compress-threads=NUMBER-OF-THREADS] [--compress-chu
 innobackupex --apply-log [--use-memory=B]\n\
              [--defaults-file=MY.CNF]\n\
              [--export] [--ibbackup=IBBACKUP-BINARY]\n\
-             [--innodb-force-recovery=1]\n\
              BACKUP-DIR\n\
 \n\
 innobackupex --copy-back [--defaults-file=MY.CNF] [--defaults-group=GROUP-NAME] BACKUP-DIR\n\
@@ -740,11 +730,10 @@ indicates an error.\n");
 
 static
 my_bool
-ibx_get_one_option(int optid,
-		const struct my_option *opt __attribute__((unused)),
-		char *argument)
+ibx_get_one_option(const struct my_option *opt,
+                   const char *argument, const char *)
 {
-	switch(optid) {
+	switch(opt->id) {
 	case '?':
 		usage();
 		exit(0);
@@ -783,17 +772,7 @@ ibx_get_one_option(int optid,
 		xtrabackup_compress = TRUE;
 		break;
 	case 'p':
-		if (argument)
-		{
-		        char *start = argument;
-			my_free(opt_ibx_password);
-			opt_ibx_password= my_strdup(argument, MYF(MY_FAE));
-			/*  Destroy argument */
-			while (*argument)
-				*argument++= 'x';
-			if (*start)
-				start[1]=0 ;
-		}
+    opt_ibx_password= argument;
 		break;
         }
 	return(0);
