@@ -1,5 +1,5 @@
 /* Copyright (c) 2010, 2015, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2021, MariaDB
+   Copyright (c) 2011, 2020, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include "sql_view.h"                        // view_checksum
 #include "sql_table.h"                       // mysql_recreate_table
 #include "debug_sync.h"                      // DEBUG_SYNC
-#include "sql_acl.h"                         // *_ACL
 #include "sp.h"                              // Sroutine_hash_entry
 #include "sql_parse.h"                       // check_table_access
 #include "strfunc.h"
@@ -122,9 +121,9 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
       Let us try to open at least a .FRM for this table.
     */
 
-    table_list->mdl_request.init(MDL_key::TABLE,
-                                 table_list->db.str, table_list->table_name.str,
-                                 MDL_EXCLUSIVE, MDL_TRANSACTION);
+    MDL_REQUEST_INIT(&table_list->mdl_request, MDL_key::TABLE,
+                     table_list->db.str, table_list->table_name.str,
+                     MDL_EXCLUSIVE, MDL_TRANSACTION);
 
     if (lock_table_names(thd, table_list, table_list->next_global,
                          thd->variables.lock_wait_timeout, 0))
@@ -213,15 +212,9 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
       Table open failed, maybe because we run out of memory.
       Close all open tables and relaese all MDL locks
     */
-#if MYSQL_VERSION < 100500
-    tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED,
-                     table->s->db.str, table->s->table_name.str,
-                     TRUE);
-#else
     tdc_release_share(share);
     share->tdc->flush(thd, true);
     share= 0;
-#endif
   }
 
   /*
@@ -447,6 +440,8 @@ dbug_err:
   */
 static bool wsrep_toi_replication(THD *thd, TABLE_LIST *tables)
 {
+  if (!WSREP(thd) || !WSREP_CLIENT(thd)) return false;
+
   LEX *lex= thd->lex;
   /* only handle OPTIMIZE and REPAIR here */
   switch (lex->sql_command)
@@ -466,17 +461,16 @@ static bool wsrep_toi_replication(THD *thd, TABLE_LIST *tables)
   /* now TOI replication, with no locks held */
   if (keys.empty())
   {
-    if (!thd->lex->no_write_to_binlog &&
-	wsrep_to_isolation_begin(thd, NULL, NULL, tables))
-      return true;
-  }
-  else
-  {
-    if (!thd->lex->no_write_to_binlog &&
-	wsrep_to_isolation_begin(thd, NULL, NULL, tables, NULL, &keys))
+    WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, tables);
+  } else {
+    WSREP_TO_ISOLATION_BEGIN_FK_TABLES(NULL, NULL, tables, &keys) {
       return true;
+    }
   }
   return false;
+
+ wsrep_error_label:
+  return true;
 }
 #endif /* WITH_WSREP */
 
@@ -498,8 +492,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                               int (handler::*operator_func)(THD *,
                                                             HA_CHECK_OPT *),
                               int (view_operator_func)(THD *, TABLE_LIST*,
-                                                       HA_CHECK_OPT *),
-                              bool is_cmd_replicated)
+                                                       HA_CHECK_OPT *))
 {
   TABLE_LIST *table;
   List<Item> field_list;
@@ -510,8 +503,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   int compl_result_code;
   bool need_repair_or_alter= 0;
   wait_for_commit* suspended_wfc;
-  bool is_table_modified= false;
-
   DBUG_ENTER("mysql_admin_table");
   DBUG_PRINT("enter", ("extra_open_options: %u", extra_open_options));
 
@@ -551,13 +542,10 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   for (table= tables; table; table= table->next_local)
     table->table= NULL;
 #ifdef WITH_WSREP
-  if (WSREP(thd))
+  if (wsrep_toi_replication(thd, tables))
   {
-    if(wsrep_toi_replication(thd, tables))
-    {
-      WSREP_INFO("wsrep TOI replication of has failed.");
-      goto err;
-    }
+    WSREP_INFO("wsrep TOI replication of has failed, skipping OPTIMIZE");
+    goto err;
   }
 #endif /* WITH_WSREP */
 
@@ -571,8 +559,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     bool open_for_modify= org_open_for_modify;
 
     DBUG_PRINT("admin", ("table: '%s'.'%s'", db, table->table_name.str));
-    DEBUG_SYNC(thd, "admin_command_kill_before_modify");
-
     strxmov(table_name, db, ".", table->table_name.str, NullS);
     thd->open_options|= extra_open_options;
     table->lock_type= lock_type;
@@ -587,14 +573,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                                 : lock_type >= TL_WRITE_ALLOW_WRITE
                                 ? MDL_SHARED_WRITE : MDL_SHARED_READ);
 
-    if (thd->check_killed())
-    {
-      open_error= false;
-      fatal_error= true;
-      result_code= HA_ADMIN_FAILED;
-      goto send_result;
-    }
-
     /* open only one table from local list of command */
     while (1)
     {
@@ -630,8 +608,9 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       close_thread_tables(thd);
       table->table= NULL;
       thd->release_transactional_locks();
-      table->mdl_request.init(MDL_key::TABLE, table->db.str, table->table_name.str,
-                              MDL_SHARED_NO_READ_WRITE, MDL_TRANSACTION);
+      MDL_REQUEST_INIT(&table->mdl_request, MDL_key::TABLE, table->db.str,
+                       table->table_name.str, MDL_SHARED_NO_READ_WRITE,
+                       MDL_TRANSACTION);
     }
 
 #ifdef WITH_PARTITION_STORAGE_ENGINE
@@ -890,8 +869,9 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       close_thread_tables(thd);
       table->table= NULL;
       thd->release_transactional_locks();
-      table->mdl_request.init(MDL_key::TABLE, table->db.str, table->table_name.str,
-                              MDL_SHARED_NO_READ_WRITE, MDL_TRANSACTION);
+      MDL_REQUEST_INIT(&table->mdl_request, MDL_key::TABLE, table->db.str,
+                       table->table_name.str, MDL_SHARED_NO_READ_WRITE,
+                       MDL_TRANSACTION);
       table->mdl_request.set_type(MDL_SHARED_READ);
 
       table->lock_type= TL_READ;
@@ -912,7 +892,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
             enum enum_field_types type= (*field_ptr)->type();
             if (type < MYSQL_TYPE_MEDIUM_BLOB ||
                 type > MYSQL_TYPE_BLOB)
-              tab->field[fields]->register_field_in_read_map();
+              bitmap_set_bit(tab->read_set, fields);
             else
               push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                                   ER_NO_EIS_FOR_FIELD,
@@ -940,7 +920,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
             enum enum_field_types type= tab->field[pos]->type();
             if (type < MYSQL_TYPE_MEDIUM_BLOB ||
                 type > MYSQL_TYPE_BLOB)
-              tab->field[pos]->register_field_in_read_map();
+              bitmap_set_bit(tab->read_set, pos);
             else
               push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                                   ER_NO_EIS_FOR_FIELD,
@@ -1115,9 +1095,7 @@ send_result_message:
                  *save_next_global= table->next_global;
       table->next_local= table->next_global= 0;
 
-      tmp_disable_binlog(thd); // binlogging is done by caller if wanted
       result_code= admin_recreate_table(thd, table);
-      reenable_binlog(thd);
       trans_commit_stmt(thd);
       trans_commit(thd);
       close_thread_tables(thd);
@@ -1233,15 +1211,15 @@ send_result_message:
         break;
       }
     }
-    /*
-      Admin commands acquire table locks and these locks are not detected by
-      parallel replication deadlock detection-and-handling mechanism. Hence
-      they must be marked as DDL so that they are not scheduled in parallel
-      with conflicting DMLs resulting in deadlock.
-    */
-    thd->transaction.stmt.mark_executed_table_admin_cmd();
     if (table->table && !table->view)
     {
+      /*
+        Don't skip flushing if we are collecting EITS statistics.
+      */
+      const bool skip_flush=
+        (operator_func == &handler::ha_analyze) && 
+        (table->table->file->ha_table_flags() & HA_ONLINE_ANALYZE) &&
+        !collect_eis;
       if (table->table->s->tmp_table)
       {
         /*
@@ -1251,10 +1229,9 @@ send_result_message:
         if (open_for_modify && !open_error)
           table->table->file->info(HA_STATUS_CONST);
       }
-      else if (open_for_modify || fatal_error)
+      else if ((!skip_flush && open_for_modify) || fatal_error)
       {
-        tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED,
-                         table->db.str, table->table_name.str, FALSE);
+        table->table->s->tdc->flush_unused(true);
         /*
           May be something modified. Consequently, we have to
           invalidate the query cache.
@@ -1275,9 +1252,10 @@ send_result_message:
     }
     else
     {
-      if (trans_commit_stmt(thd))
+      if (trans_commit_stmt(thd) ||
+          (stmt_causes_implicit_commit(thd, CF_IMPLICIT_COMMIT_END) &&
+           trans_commit_implicit(thd)))
         goto err;
-      is_table_modified= true;
     }
     close_thread_tables(thd);
     thd->release_transactional_locks();
@@ -1300,16 +1278,6 @@ send_result_message:
 
     if (protocol->write())
       goto err;
-    DEBUG_SYNC(thd, "admin_command_kill_after_modify");
-  }
-  if (is_table_modified && is_cmd_replicated &&
-      (!opt_readonly || thd->slave_thread) && !thd->lex->no_write_to_binlog)
-  {
-    thd->get_stmt_da()->set_overwrite_status(true);
-    auto res= write_bin_log(thd, true, thd->query(), thd->query_length());
-    thd->get_stmt_da()->set_overwrite_status(false);
-    if (res)
-      goto err;
   }
 
   my_eof(thd);
@@ -1372,7 +1340,7 @@ bool mysql_assign_to_keycache(THD* thd, TABLE_LIST* tables,
   check_opt.key_cache= key_cache;
   DBUG_RETURN(mysql_admin_table(thd, tables, &check_opt,
 				"assign_to_keycache", TL_READ_NO_INSERT, 0, 0,
-				0, 0, &handler::assign_to_keycache, 0, false));
+				0, 0, &handler::assign_to_keycache, 0));
 }
 
 
@@ -1399,7 +1367,7 @@ bool mysql_preload_keys(THD* thd, TABLE_LIST* tables)
   */
   DBUG_RETURN(mysql_admin_table(thd, tables, 0,
 				"preload_keys", TL_READ_NO_INSERT, 0, 0, 0, 0,
-				&handler::preload_keys, 0, false));
+				&handler::preload_keys, 0));
 }
 
 
@@ -1420,8 +1388,15 @@ bool Sql_cmd_analyze_table::execute(THD *thd)
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt,
                          "analyze", lock_type, 1, 0, 0, 0,
-                         &handler::ha_analyze, 0, true);
-
+                         &handler::ha_analyze, 0);
+  /* ! we write after unlocking the table */
+  if (!res && !m_lex->no_write_to_binlog && (!opt_readonly || thd->slave_thread))
+  {
+    /*
+      Presumably, ANALYZE and binlog writing doesn't require synchronization
+    */
+    res= write_bin_log(thd, TRUE, thd->query(), thd->query_length());
+  }
   m_lex->first_select_lex()->table_list.first= first_table;
   m_lex->query_tables= first_table;
 
@@ -1447,7 +1422,7 @@ bool Sql_cmd_check_table::execute(THD *thd)
 
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt, "check",
                          lock_type, 0, 0, HA_OPEN_FOR_REPAIR, 0,
-                         &handler::ha_check, &view_check, false);
+                         &handler::ha_check, &view_check);
 
   m_lex->first_select_lex()->table_list.first= first_table;
   m_lex->query_tables= first_table;
@@ -1472,8 +1447,15 @@ bool Sql_cmd_optimize_table::execute(THD *thd)
     mysql_recreate_table(thd, first_table, true) :
     mysql_admin_table(thd, first_table, &m_lex->check_opt,
                       "optimize", TL_WRITE, 1, 0, 0, 0,
-                      &handler::ha_optimize, 0, true);
-
+                      &handler::ha_optimize, 0);
+  /* ! we write after unlocking the table */
+  if (!res && !m_lex->no_write_to_binlog && (!opt_readonly || thd->slave_thread))
+  {
+    /*
+      Presumably, OPTIMIZE and binlog writing doesn't require synchronization
+    */
+    res= write_bin_log(thd, TRUE, thd->query(), thd->query_length());
+  }
   m_lex->first_select_lex()->table_list.first= first_table;
   m_lex->query_tables= first_table;
 
@@ -1496,8 +1478,16 @@ bool Sql_cmd_repair_table::execute(THD *thd)
                          TL_WRITE, 1,
                          MY_TEST(m_lex->check_opt.sql_flags & TT_USEFRM),
                          HA_OPEN_FOR_REPAIR, &prepare_for_repair,
-                         &handler::ha_repair, &view_repair, true);
+                         &handler::ha_repair, &view_repair);
 
+  /* ! we write after unlocking the table */
+  if (!res && !m_lex->no_write_to_binlog && (!opt_readonly || thd->slave_thread))
+  {
+    /*
+      Presumably, REPAIR and binlog writing doesn't require synchronization
+    */
+    res= write_bin_log(thd, TRUE, thd->query(), thd->query_length());
+  }
   m_lex->first_select_lex()->table_list.first= first_table;
   m_lex->query_tables= first_table;
 
