@@ -249,8 +249,14 @@ static int open_table(THD* thd,
   tables.init_one_table(schema_name,
                         table_name,
                         NULL, lock_type);
+  thd->lex->query_tables_own_last= 0;
 
   if (!open_n_lock_single_table(thd, &tables, tables.lock_type, flags)) {
+    if (thd->is_error()) {
+      WSREP_WARN("Can't lock table %s.%s : %d (%s)",
+                 schema_name->str, table_name->str,
+                 thd->get_stmt_da()->sql_errno(), thd->get_stmt_da()->message());
+    }
     close_thread_tables(thd);
     my_error(ER_NO_SUCH_TABLE, MYF(0), schema_name->str, table_name->str);
     DBUG_RETURN(1);
@@ -868,7 +874,7 @@ Wsrep_view Wsrep_schema::restore_view(THD* thd, const Wsrep_id& own_id) const {
       close_thread_tables(thd);
     }
   }
-  thd->mdl_context.release_transactional_locks();
+  thd->release_transactional_locks();
 
   thd->variables.wsrep_sync_wait= wsrep_sync_wait_saved;
 
@@ -912,6 +918,13 @@ int Wsrep_schema::append_fragment(THD* thd,
               thd->thread_id,
               os.str().c_str(),
               transaction_id.get());
+  /* use private query table list for the duration of fragment storing,
+     populated query table list from "parent DML" may cause problems .e.g
+     for virtual column handling
+ */
+  Query_tables_list query_tables_list_backup;
+  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
+
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   Wsrep_schema_impl::init_stmt(thd);
 
@@ -919,6 +932,7 @@ int Wsrep_schema::append_fragment(THD* thd,
   if (Wsrep_schema_impl::open_for_write(thd, sr_table_str.c_str(), &frag_table))
   {
     trans_rollback_stmt(thd);
+    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
@@ -932,9 +946,11 @@ int Wsrep_schema::append_fragment(THD* thd,
   if ((error= Wsrep_schema_impl::insert(frag_table))) {
     WSREP_ERROR("Failed to write to frag table: %d", error);
     trans_rollback_stmt(thd);
+    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
   Wsrep_schema_impl::finish_stmt(thd);
+  thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
   DBUG_RETURN(0);
 }
 
@@ -951,15 +967,23 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
               ws_meta.seqno().get());
   DBUG_ASSERT(ws_meta.seqno().is_undefined() == false);
 
+  /* use private query table list for the duration of fragment storing,
+     populated query table list from "parent DML" may cause problems .e.g
+     for virtual column handling
+ */
+  Query_tables_list query_tables_list_backup;
+  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
+
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   int error;
-  uchar key[MAX_KEY_LENGTH];
+  uchar key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
   key_part_map key_map= 0;
   TABLE* frag_table= 0;
 
   Wsrep_schema_impl::init_stmt(thd);
   if (Wsrep_schema_impl::open_for_write(thd, sr_table_str.c_str(), &frag_table))
   {
+    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
@@ -980,6 +1004,7 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
                  error);
     }
     Wsrep_schema_impl::finish_stmt(thd);
+    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
@@ -995,11 +1020,13 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
                 frag_table->s->table_name.str,
                 error);
     Wsrep_schema_impl::finish_stmt(thd);
+    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
   int ret= Wsrep_schema_impl::end_index_scan(frag_table);
   Wsrep_schema_impl::finish_stmt(thd);
+  thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
   DBUG_RETURN(ret);
 }
 
@@ -1015,7 +1042,7 @@ static int remove_fragment(THD*                  thd,
               seqno.get());
   int ret= 0;
   int error;
-  uchar key[MAX_KEY_LENGTH];
+  uchar key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
   key_part_map key_map= 0;
 
   DBUG_ASSERT(server_id.is_undefined() == false);
@@ -1141,7 +1168,7 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
   int ret= 1;
   int error;
   TABLE* frag_table= 0;
-  uchar key[MAX_KEY_LENGTH];
+  uchar key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
   key_part_map key_map= 0;
 
   for (std::vector<wsrep::seqno>::const_iterator i= fragments.begin();
